@@ -1,10 +1,12 @@
 
 '''
 Advanced Power Monitor
-2008-2011(c) Kirill Plyashkevich <ru.spirit@gmail.com>
+2008-2012(c) Kirill Plyashkevich <ru.spirit@gmail.com>
 Connects to bunch of dbus signals, monitors values change and emmits signal in case of update.
 '''
 
+from __future__ import with_statement
+import threading
 import random
 import gobject
 import os.path
@@ -19,7 +21,7 @@ from advpowcommon.util.common import *
 class apm(dbus.service.Object):
 
   appname = 'Advanced Power Monitor'
-  version = '0.6.0-4'
+  version = '0.6.0-6'
   iface = "ru.spirit.advanced_power_monitor"
   opath = "/ru/spirit/advanced_power_monitor"
   wrappers_root = 'wrappers'
@@ -28,6 +30,7 @@ class apm(dbus.service.Object):
   def __init__(self):
     print '%s %s' % (self.appname, self.version)
     print '%s\n' % (device.get_property('OSSO_VERSION'))
+    self.__lock = threading.Lock()
     self.__bus = dbus.SystemBus(mainloop = DBusGMainLoop(), private = True)
     dbus.service.Object.__init__(self, dbus.service.BusName(self.iface, self.__bus), self.opath)
 
@@ -55,19 +58,18 @@ class apm(dbus.service.Object):
         module_class = ClassLoader.get_module_class(m)
         if module_class:
           class_list.append(module_class)
-      if class_list:
-        for ref_module in class_list:
-          c_module = ref_module()
-          if c_module.is_available():
-            output_loaded = '%s %s' % (output_loaded, c_module.get_name())
-            item = {'value': c_module.value, 'time': c_module.interval,
-                    'object': c_module, 'async': c_module.async,
-                    'clients': set()}
-            self.__info_map[k.replace('%s.' % (self.modules_root), '')] = item
-            loaded_counter += 1
-            break
-          else:
-            del c_module
+      for ref_module in class_list:
+        c_module = ref_module()
+        if c_module.is_available():
+          output_loaded = '%s %s' % (output_loaded, c_module.get_name())
+          item = {'value': c_module.value, 'time': c_module.interval,
+                  'object': c_module, 'async': c_module.async,
+                  'clients': set()}
+          self.__info_map[k.replace('%s.' % (self.modules_root), '')] = item
+          loaded_counter += 1
+          break
+        else:
+          del c_module
     print 'Loaded modules (%s/%s):%s\n' % (loaded_counter, len(modules), output_loaded)
 
     print 'Loading wrappers...'
@@ -98,21 +100,23 @@ class apm(dbus.service.Object):
     self.__cleanup()
 
   def __cleanup(self, *args):
-    active_wrappers = self.__wrappers.values()
-    for wr in active_wrappers:
-      try:
-        if wr['object']:
-          wr['object']._cleanup()
-      except KeyError:
-        pass
-    for x in self.__info_map:
-      try:
-        gobject.source_remove(self.__info_map[x]['src_evt'])
-      except KeyError:
-        pass
-    self.__destroy_interfaces()
-    self.__destroy_signals()
-    self.__working = False
+    with self.__lock:
+      active_wrappers = self.__wrappers.values()
+      for wr in active_wrappers:
+        try:
+          if wr['object']:
+            wr['object']._cleanup()
+        except KeyError:
+          pass
+      info_values = self.__info_map.values()
+      for val in info_values:
+        try:
+          gobject.source_remove(val['src_evt'])
+        except KeyError:
+          pass
+      self.__destroy_interfaces()
+      self.__destroy_signals()
+      self.__working = False
     print 'EXIT'
 #    sys.exit(0)
 
@@ -124,7 +128,12 @@ class apm(dbus.service.Object):
   def __add_signal(self, signal):
     self.__remove_signal(signal)
     try:
-      self.__signals[signal]['obj'] = self.__bus.add_signal_receiver(getattr(self, 'handle_%s' % signal), signal_name=self.__signals[signal]['signal_name'], dbus_interface=self.__signals[signal]['dbus_interface'], path=self.__signals[signal]['path'])
+      signal_obj = self.__signals[signal]
+      signal_obj['obj'] = self.__bus.add_signal_receiver(
+        getattr(self, 'handle_%s' % signal),
+        signal_name=signal_obj['signal_name'],
+        dbus_interface=signal_obj['dbus_interface'],
+        path=signal_obj['path'])
     except Exception, e:
       self.__print_exception('__add_signal', str(e))
 
@@ -135,7 +144,7 @@ class apm(dbus.service.Object):
   def __remove_signal(self, key):
     try:
       c_signal = self.__signals[key].pop('obj')
-      self.__bus.remove_signal_receiver(c_signal)
+      self.__bus.remove_signal_receiver(c_signal, signal_name=self.__signals[key]['signal_name'], dbus_interface=self.__signals[key]['dbus_interface'], path=self.__signals[key]['path'])
       c_signal.remove()
     except KeyError:
       pass
@@ -195,27 +204,28 @@ class apm(dbus.service.Object):
 
   @dbus.service.method(dbus_interface='ru.spirit.advanced_power_monitor.request', in_signature='ss')
   def change_wrapper(self, key, new_wrapper):
-    wrapper_object = None
-    try:
-      wrapper_object = self.__wrappers[key]['list'][self.__wrappers[key]['available'][new_wrapper]](self.__bus, True)
-    except KeyError:
-      return
-    new_props = wrapper_object.get_properties()
-    clients = dict.fromkeys(new_props.keys(), set())
-    if new_wrapper != self.__wrappers[key]['name'] and wrapper_object.is_available():#new_wrapper in self.__wrappers[key]['available'].keys()
-      if self.__wrappers[key]['object']:
-        props = self.__wrappers[key]['object'].get_properties()
-        for pkey in props:
-          if pkey in new_props:
-            clients[pkey] = self.__info_map[pkey]['clients']
-          del self.__info_map[pkey]
-        self.__wrappers[key]['object']._cleanup()
-      self.__wrappers[key]['name'] = new_wrapper
-      self.__wrappers[key]['object'] = wrapper_object
-      for pkey in new_props:
-        self.__info_map[pkey] = {'value': new_props[pkey], 'async': True, 'clients': clients[pkey]}
-      self.__init_wrapper(key)
-      self.__wrappers[key]['object'].finish_init()
+    with self.__lock:
+      wrapper_object = None
+      try:
+        wrapper_object = self.__wrappers[key]['list'][self.__wrappers[key]['available'][new_wrapper]](self.__bus, True)
+      except KeyError:
+        return
+      new_props = wrapper_object.get_properties()
+      clients = dict.fromkeys(new_props.keys(), set())
+      if new_wrapper != self.__wrappers[key]['name'] and wrapper_object.is_available():#new_wrapper in self.__wrappers[key]['available'].keys()
+        if self.__wrappers[key]['object']:
+          props = self.__wrappers[key]['object'].get_properties()
+          for pkey in props:
+            if pkey in new_props:
+              clients[pkey] = self.__info_map[pkey]['clients']
+            del self.__info_map[pkey]
+          self.__wrappers[key]['object']._cleanup()
+        self.__wrappers[key]['name'] = new_wrapper
+        self.__wrappers[key]['object'] = wrapper_object
+        for pkey in new_props:
+          self.__info_map[pkey] = {'value': new_props[pkey], 'async': True, 'clients': clients[pkey]}
+        self.__init_wrapper(key)
+        self.__wrappers[key]['object'].finish_init()
 
   @dbus.service.method(dbus_interface='ru.spirit.advanced_power_monitor.request', in_signature='s', out_signature='as')
   def get_available_wrappers(self, key):
