@@ -1,6 +1,6 @@
 '''
 Advanced Power Abstract Wrappers
-2008-2011(c) Kirill Plyashkevich <ru.spirit@gmail.com>
+2008-2012(c) Kirill Plyashkevich <ru.spirit@gmail.com>
 '''
 
 import dbus
@@ -151,13 +151,11 @@ class AbstractWrapper():
       self.__init_dbus_signal(key)
 
   def __init_dbus_signal(self, key):
+    self.__destroy_dbus_signal(key)
     try:
-      if not 'receiver_match' in self.__dbus_signals[key]:
-        self.__dbus_signals[key]['receiver_match'] = self.__bus.add_signal_receiver(
-        getattr(self, 'handle_%s_%s' % (self.__type, key)), 
-        signal_name=self.__dbus_signals[key]['source'][2], 
-        dbus_interface=self.__dbus_signals[key]['source'][1], 
-        path=self.__dbus_signals[key]['source'][0])
+      self.__dbus_signals[key]['match'] = self.__bus.add_signal_receiver(
+        getattr(self, 'handle_%s_%s' % (self.__type, key)),
+        **self.__dbus_signals[key])
     except Exception, e:
       print e
 
@@ -166,6 +164,7 @@ class AbstractWrapper():
       self.__destroy_dbus_signal(key)
 
   def __destroy_dbus_signal(self, key):
-    receiver_match = self.__dbus_signals[key].pop('receiver_match')
-    self.__bus.remove_signal_receiver(receiver_match)
-    receiver_match.remove()
+    if 'match' in self.__dbus_signals[key]:
+      receiver_match = self.__dbus_signals[key].pop('match')
+      self.__bus.remove_signal_receiver(receiver_match,**self.__dbus_signals[key])
+      receiver_match.remove()
