@@ -1,6 +1,6 @@
 '''
 Advanced Power Battery Wraper BME
-2009-2011(c) Kirill Plyashkevich <ru.spirit@gmail.com>
+2009-2012(c) Kirill Plyashkevich <ru.spirit@gmail.com>
 '''
 
 from advpowcommon.util.execn import *
@@ -19,11 +19,18 @@ class BatteryBME(AbstractBattery):
     self.interfaces_map = {'BME': ('org.freedesktop.Hal', self.bme_path, self.bme_iface)}
 
   def get_default_dbus_signals(self):
-    return {'charger_is_connected': {'source': (self.bme_signal_path, self.bme_signal_iface, 'charger_charging_on')},
-            'charger_is_disconnected': {'source': (self.bme_signal_path, self.bme_signal_iface, 'charger_disconnected')},
-            'is_full': {'source': (self.bme_signal_path, self.bme_signal_iface, 'battery_full')},
-            'status': {'source': (self.bme_path, self.bme_iface, 'PropertyModified')},
-           }
+    return {'charger_is_connected': {'path': self.bme_signal_path,
+                                     'dbus_interface': self.bme_signal_iface,
+                                     'signal_name': 'charger_charging_on'},
+            'charger_is_disconnected': {'path': self.bme_signal_path,
+                                        'dbus_interface': self.bme_signal_iface,
+                                        'signal_name': 'charger_disconnected'},
+            'is_full': {'path': self.bme_signal_path,
+                        'dbus_interface': self.bme_signal_iface,
+                        'signal_name': 'battery_full'},
+            'status': {'path': self.bme_path,
+                       'dbus_interface': self.bme_iface,
+                       'signal_name': 'PropertyModified'}}
 
   def get_friendly_name(self):
     return 'Nokia\'s BME'
